#include <avr/io.h>
#include <stdint.h>
#include <util/twi.h>
#include "i2c.h"
#include "general.h"
#include "other.h"
#define SLA_W	0b11010000
#define SLA_R	0b11010001
#define TIMEOUT_WAIT	0xFFUL		// So hoch wird gezhlt, bis Timeout erreicht wird


/*******************************************************
	Funktion :		Initialisiere I2C
	Eigenschaften:	100 kHz Clock
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void i2c_init(void)
{
  // TWI Clock = 100 kHz, TWPS = 0 (Prescaler 1), TWBR mit SCL_Clock berechnen
  TWSR = (0<<TWPS1) | (0<<TWPS0);    // Prescaler 1
  TWBR = ((F_OSC/SCL_CLOCK)-16)/2;   // => 100 kHz Clock
}


/*******************************************************
	Funktion :		Error-Funktion
	Eigenschaften:	Ruft allgemeine Error Routine auf
	bergabewert:	1 Byte Error Code
	Rckgabewert:	-
********************************************************/
void i2c_error(uint8_t error_code)
{
	// ERROR-Routine
	programm_error(error_code);
}


/*******************************************************
	Funktion :		I2C sende Daten
	Eigenschaften:	Sende 1 Byte ber Bus
	bergabewert:	1 Byte Daten
	Rckgabewert:	-
********************************************************/
void i2c_send(uint8_t data)
{
	uint8_t twst;									// Variable fr Statuscode

	TWDR = data;									// Gib Daten in bertragungspuffer
	TWCR = (1<<TWINT) | (1<<TWEN);					// Sende Daten
	while(!(TWCR & (1<<TWINT)));					// Warte bis TWINT gesetzt ist(Also senden fertig)

	twst = TWSR & 0xF8;								// Maskiere Statuscode
	if((twst != TW_MT_DATA_ACK) && (twst != TW_MR_SLA_ACK) && (twst != TW_MT_SLA_ACK))	// Status Register richten Status? Ansonsten ERROR
	{
		i2c_error(EC_I2C_SEND);						
	}
}


/*******************************************************
	Funktion :		I2C Start
	Eigenschaften:	Startet I2C mit einem Gert
	bergabewert:	1 Byte Device Addresse
	Rckgabewert:	-
********************************************************/
void i2c_start(uint8_t devaddr)
{
	uint8_t twst;									// Variable fr Statuscode

	TWCR = (1 << TWINT) | (1<<TWSTA) | (1<<TWEN);	// Sende START Signal bzw. REP-Start
	while(!(TWCR & (1<<TWINT)));					// Warte bis TWINT gesetzt ist (also START gesendet)

	twst = TWSR & 0xF8;								// Maskiere Statuscode
	if((twst != TW_START) && (twst != TW_REP_START))// Status Register richten Status? Ansonsten ERROR
	{
		i2c_error(EC_I2C_START);									
	}

	i2c_send(devaddr);								// Sende Device Addresse
}



/*******************************************************
	Funktion :		I2C Stop
	Eigenschaften:	Stopt die I2C Verbindung
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void i2c_stop(void)
{
	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);		// STOP Signal
}



/*******************************************************
	Funktion :		Hole Daten von I2C
	Eigenschaften:	Holt Daten + ACK
	bergabewert:	-
	Rckgabewert:	1 Byte Daten
********************************************************/
uint8_t i2c_get_ack(void)
{
	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWEA);	// ACK

	timeout_counter = 0;					// Setze sicherheitshalber zurck
	while(timeout_counter != TIMEOUT_WAIT)	// Timeout Schleife
	{
		if(TWCR & (1<<TWINT))				// bertragungspuffer leer?
		{	
			timeout_counter = 0;			// Setze wieder zurck (fr nchste berprfung
			return TWDR;					// Return Daten
		}
		else
		{
			timeout_counter++;
		}
	}
	timeout_counter = 0;					// Setze wieder zurck (fr nchste berprfung
	i2c_error(EC_I2C_GET_ACK);				// Rufe Error-Funktion mit Errorcode auf
	return -1;								// Return Fehler
}
																		
																				

/*******************************************************
	Funktion :		Hole Daten von I2C
	Eigenschaften:	Holt Daten + NACK
	bergabewert:	-
	Rckgabewert:	1 Byte Daten
********************************************************/
uint8_t i2c_get_nack(void)
{
	TWCR = (1<<TWINT) | (1<<TWEN);			// NACK

	timeout_counter = 0;					// Setze sicherheitshalber zurck
	while(timeout_counter != TIMEOUT_WAIT)	// Timeout Schleife
	{
		if(TWCR & (1<<TWINT))				// bertragungspuffer leer?
		{	
			timeout_counter = 0;			// Setze wieder zurck (fr nchste berprfung
			return TWDR;					// Return Daten
		}
		else
		{
			timeout_counter++;
		}
	}
	timeout_counter = 0;					// Setze wieder zurck (fr nchste berprfung
	i2c_error(EC_I2C_GET_ACK);				// Rufe Error-Funktion mit Errorcode auf
	return -1;								// return die Daten
}



/*******************************************************
	Funktion :		Sende die Zeit an RTC
	Eigenschaften:	-
	bergabewert:	-
	Rckgabewert:	struct time
********************************************************/
void i2c_send_time(struct time t)
{
	t = bcd_to_rtc_values(t);			// Umwandlen in RTC Werte

	// Sende die aktuelle Zeit zum RTC
	i2c_start(SLA_W);					// START + Write
	i2c_send(0x00);						// Registeraddresse auf 0 Stellen
	i2c_send(t.seconds);	
	i2c_send(t.minutes);
	i2c_send(t.hours);
	i2c_send(0x02);						// Irgendwas, hat keine Auswirkung (Wochentag)
	i2c_send(t.date);
	i2c_send(t.month);
	i2c_send(t.year);
	i2c_stop();							// STOP	
}



/*******************************************************
	Funktion :		Hole die Zeit vom RTC
	Eigenschaften:	-
	bergabewert:	struct time
	Rckgabewert:	-
********************************************************/
struct time i2c_get_time(void)
{
	struct time t;

	i2c_start(SLA_W);					// START mit Write
	i2c_send(0x00);						// Setze Registeraddresse 00
	i2c_start(SLA_R);					// REPSTART mit Read
	t.seconds = i2c_get_ack();
	t.minutes = i2c_get_ack();
	t.hours = i2c_get_ack();
	i2c_get_ack();						// Day wird nicht bentigt
	t.date = i2c_get_ack();
	t.month = i2c_get_ack();
	t.year = i2c_get_nack();
	i2c_stop();
	
	t = rtc_values_to_bcd(t);			// Umwandeln in BCD
	return t;
}
