#include <avr/io.h>
#include <stdint.h>
#include "other.h"
#include "usart.h"
#include "hd44780.h"
#include <avr/pgmspace.h>
#include "registerstrings.h"
#include "joystick.h"
#include <stdlib.h>
#include <util/delay.h>

/*******************************************************
	Funktion :		Get Sekunden
	Eigenschaften:	Liefere Anzahl der Sekunden von Time zurck
	bergabewert:	struct time
	Rckgabewert:	4 Byte
********************************************************/
uint32_t get_sec(struct time t)
{
	// Berechne Sekunden eines Jahres
	// Jahre*12 + Monate = Gesamte Monate
	// Gesamten Monate * 31 Tage + Tage = Gesamte Tage 
	// 31 kann genommen werden, da nur Differenz und nicht genau Anzahl an Sekunden bentigt wird
	// Genauso kann ein Schaltjahr und die Schaltsekunde vernachlssigt werden
	// Gesamte Tage * 24 Stunden + Stunden = Gesamte Stunden
	// Gesamte Stunden * 60 + Minuten = Gesamte Minuten
	// Gesamte Minuten * 60 + Sekunden = Gesamte Sekunden
	// Return Gesamte Sekunden
	return(((((t.year * 12UL + t.month) * 31 + t.date) * 24 + t.hours) * 60 + t.minutes) * 60 + t.seconds);
}


/*******************************************************
	Funktion :		Liegt aktuelle Zeit im Soll-Messbereich?
	Eigenschaften:	1... Ja, 0...Nein
	bergabewert:	-
	Rckgabewert:	1 Byte
********************************************************/
uint8_t time_in_target_area(struct time rtc, struct time start, struct time end)
{
	// Wandle aktuelle Zeit (von RTC) in Sekunden um
	uint32_t now_sec = get_sec(rtc);

	// Aktuelle Zeit grer als Start und kleiner als End => logisch 1, ansonsten => logisch 0
	return (now_sec > get_sec(start) && now_sec < get_sec(end));
}



/*******************************************************
	Funktion :		Wandelt von RTC zu BCD Werte um
	Eigenschaften:	-
	bergabewert:	struct time
	Rckgabewert:	struct time
********************************************************/
struct time rtc_values_to_bcd(struct time t)
{
	uint8_t bcd_temp_low;						// Zwischenspeicher fr Einerstelle
	uint8_t bcd_temp_high;						// Zwischenspeicher fr Zehnerstelle

	// Sekunden in BCD umwandeln
	bcd_temp_low = t.seconds & 0b00001111;		// Maskiere Einerstelle
	bcd_temp_high = t.seconds & 0b01110000;		// Maskiere 10er Stelle
	bcd_temp_high = bcd_temp_high >> 4;			// Verschiebe um 4 nach rechts
	t.seconds = bcd_temp_high * 10 + bcd_temp_low;

	// Minuten in BCD umwandeln
	bcd_temp_low = t.minutes & 0b00001111;		// Maskiere Einerstelle
	bcd_temp_high = t.minutes & 0b01110000;		// Maskiere 10er Stelle
	bcd_temp_high = bcd_temp_high >> 4;			// Verschiebe um 4 nach rechts
	t.minutes = bcd_temp_high * 10 + bcd_temp_low;

	// Stunden in BCD umwandeln
	bcd_temp_low = t.hours & 0b00001111;		// Maskiere Einerstelle
	bcd_temp_high = t.hours & 0b00110000;		// Maskiere 10er Stelle
	bcd_temp_high = bcd_temp_high >> 4;			// Verschiebe um 4 nach rechts
	t.hours = bcd_temp_high * 10 + bcd_temp_low;

	// Tag in BCD umwandeln
	bcd_temp_low = t.date & 0b00001111;			// Maskiere Einerstelle
	bcd_temp_high = t.date & 0b00110000;		// Maskiere 10er Stelle
	bcd_temp_high = bcd_temp_high >> 4;			// Verschiebe um 4 nach rechts
	t.date = bcd_temp_high * 10 + bcd_temp_low;

	// Monate in BCD umwandeln
	bcd_temp_low = t.month & 0b00001111;		// Maskiere Einerstelle
	bcd_temp_high = t.month & 0b00010000;		// Maskiere 10er Stelle
	bcd_temp_high = bcd_temp_high >> 4;			// Verschiebe um 4 nach rechts
	t.month = bcd_temp_high * 10 + bcd_temp_low;

	// Jahr in BCD umwandeln
	bcd_temp_low = t.year & 0b00001111;			// Maskiere Einerstelle
	bcd_temp_high = t.year & 0b11110000;		// Maskiere 10er Stelle
	bcd_temp_high = bcd_temp_high >> 4;			// Verschiebe um 4 nach rechts
	t.year = bcd_temp_high * 10 + bcd_temp_low;

	return t;
}



/*******************************************************
	Funktion :		Wandelt von BCD zu RTC Werte um
	Eigenschaften:	-
	bergabewert:	struct time
	Rckgabewert:	struct time
********************************************************/
struct time bcd_to_rtc_values(struct time t)
{
	uint8_t bcd_temp_low;								// Temporrer Speicher fr Einerstelle
	uint8_t bcd_temp_high;								// Temporrer Speicher fr Zehnerstelle

	// Sekunden in RTC Werte umwandeln
	bcd_temp_low = t.seconds % 10;						// Berechne Einerstelle
	bcd_temp_high = (t.seconds - bcd_temp_low) / 10;	// Berechne Zehnerstelle
	bcd_temp_high = bcd_temp_high << 4;
	t.seconds = bcd_temp_low | bcd_temp_high;

	// Minuten in RTC Werte umwandeln
	bcd_temp_low = t.minutes % 10;						// Berechne Einerstelle
	bcd_temp_high = (t.minutes - bcd_temp_low) / 10;	// Berechne Zehnerstelle
	bcd_temp_high = bcd_temp_high << 4;
	t.minutes = bcd_temp_low | bcd_temp_high;

	// Stunden in RTC Werte umwandeln
	bcd_temp_low = t.hours % 10;						// Berechne Einerstelle
	bcd_temp_high = (t.hours - bcd_temp_low) / 10;		// Berechne Zehnerstelle
	bcd_temp_high = bcd_temp_high << 4;
	t.hours = bcd_temp_low | bcd_temp_high;

	// Tag in RTC Werte umwandeln
	bcd_temp_low = t.date % 10;							// Berechne Einerstelle
	bcd_temp_high = (t.date - bcd_temp_low) / 10;		// Berechne Zehnerstelle
	bcd_temp_high = bcd_temp_high << 4;
	t.date = bcd_temp_low | bcd_temp_high;

	// Monat in RTC Werte umwandeln
	bcd_temp_low = t.month % 10;						// Berechne Einerstelle
	bcd_temp_high = (t.month - bcd_temp_low) / 10;		// Berechne Zehnerstelle
	bcd_temp_high = bcd_temp_high << 4;
	t.month = bcd_temp_low | bcd_temp_high;

	// Jahr in RTC Werte umwandeln
	bcd_temp_low = t.year % 10;							// Berechne Einerstelle
	bcd_temp_high = (t.year - bcd_temp_low) / 10;		// Berechne Zehnerstelle
	bcd_temp_high = bcd_temp_high << 4;
	t.year = bcd_temp_low | bcd_temp_high;
	
	return t;
}

void programm_error(uint8_t error_code)
{
	errorcode = error_code;					// Stelle Errorcode auch fr globale Variable ein
	display_write_text("Status:",errorcode);
}

// VIELLEICHT GLOBALE ERRORCODE LSCHEN??? WIRKLICH NOTWENDIG???

